function [errors, norm]=sroc_err(d,d_valid,GT,G,C,O,delta,zone);
%
%  [errors, norm]=sroc_err(d,d_valid,GT,G,C,O,delta,zone)
%
%       Computes various disparity map errors.
%
%
%     Input:
%        d - disparity map to be analyzed
%  d_valid - mask defining the valid part of disparity map
%       GT - ground-truth disparity map
%
%        G - area of the GroundTruth zone
%        C - area of the symmetrical OCCLUSION zone
%        O - area of the symmetrical OCCLUSION BOUNDARY zone
%    delta - maximum allowed disparity error
%     zone - zonetype ('FX'/'X') for FN evaluation; default:'X'
%
%
%    Output:
%    errors - absolute errors: MI ... mismatches, FP ... false
%             positives, FN ... false negatives
%      norm - normalization factors: MIR ... for mismatches (MI),
%             FPR ... for FP, FNR ... for FN
%
%
% (c) Jan Cech, Jana Kostliva ({cechj,kostliva}@cmp.felk.cvut.cz)
%     CMP, FEE CTU in Prague, 23-Apr-2007
%


if ~exist('zone','var')
    zone = 'X'; %original in BenCOS2007 approach
end 
G_area = G;

% do not evaluate matches which are in the area where GT is 
% not defined
d(~d_valid) = NaN;
GT(~d_valid) = NaN;

%inicialization of errors and normalizations
errors.MI = 0;
errors.FP = 0;
errors.FN = 0;

norm.MIR = 0;
norm.FPR = 0;
norm.FNR = 0;

for row=1:size(GT,1), %evaluate each scanline
   d_ = d(row,:);
   gt_ = GT(row,:);
   
   xl = 1:length(d_);

   G = G_area{row};
   %remove ground-truth points which are invalid
   dv = find(d_valid(row,:)); 
   [tmp,i] = setdiff(G(:,1),dv);
   G(i,:) = [];
      
   Gind = [G,(1:length(G))']; %indexed G [xl,xr,ind]
   
   xr = xl-d_;
   M = [xl',xr'];
   M(isnan(xr'),:) = [];
   
   C_=C{row};
   O_=O{row};
   
   %indices into M
   m = (1:size(M,1))'; %matched
   c = isinside(M,C_); %occluded
   o = isinside(M,O_,'='); %boundary
   g = isinGT(M,gt_,delta); %in ground truth
   
   %in case the O-zone and C-zone overlap
   c = setdiff(c,o); %fixed 7.2.2007

   %indices into G
   [go,ngo] = isinside(G,O_,'='); 
   go = unique(go); %in case boundary areas overlap 
   
   %errors
   %mismatches
   MI = M;      
   MI([g;c;o],:)=[]; 
   
   %false positives
   FP = M(c,:);  

   %false negatives
   FN = Gind;
   FN([go],:)=[]; %G\O
   FN = G(FNf(FN,zone,M),:);
   
   %normalizations
   MIR_d = max(0,length(m)-length(o)-length(c));
   FPR_d = max(0,length(m)-length(o)); 
   FNR_d = length(G)-length(go);
   
   %acumulating error and normalization from each scanline
   errors.MI = errors.MI + size(MI,1);
   errors.FP =	errors.FP + size(FP,1);
   errors.FN = errors.FN + size(FN,1);

   norm.MIR = norm.MIR + MIR_d;
   norm.FPR = norm.FPR + FPR_d;
   norm.FNR = norm.FNR + FNR_d;
    
   
end


% ----------------------------------------------

%functions

function [ind,indC] = isinside(P,area,opt)
%isin - check if point is inside the rectangular area
%
% [ind,indC] = isinside(P,area)
%
%  - P......(nx2) matrix of points [xl,xr]
%  - area...(rx4) matrix of area corners [i1,i2,j1,j2]
%  - opt....'=' do not use sharp inequalities (include boundary)
%                 
%  -> ind...index into P
%     indC..complementary index into P
%

if ~exist('opt','var'),
   opt = '';
end

ind = []; 
indC = 1:size(P,1);
for r=1:size(area,1), 
   i1 = area(r,1); i2 = area(r,2);
   j1 = area(r,3); j2 = area(r,4);
   
   if strcmp(opt,'='),
      S=(P(:,1)>=i1 & P(:,1)<=i2 & P(:,2)>=j1 & P(:,2)<=j2);
      ii = find(S);
   else
      S=(P(:,1)>i1 & P(:,1)<i2 & P(:,2)>j1 & P(:,2)<j2);
      ii = find(S);
   end
   
   ind = [ind;ii];
end
indC(ind)=[];
indC=indC';
return
% ----------------------------------------------
function [ind] = isinGT(P, gt, delta)
%
% [ind] = isinGT(P, gt, delta)
%
%  P.......matched points (nx2) [xl,xr]
%  gt......ground truth scanline
%  delta...tolerance (default: 1 pixel)
%

if ~exist('delta','var'),
   delta = 1;
end

xl = P(:,1);
d  = [P(:,1) - P(:,2)];

 
ind = find(abs(d-gt(xl)') < delta);

return
% ----------------------------------------------
function [ind] = notinGT(gt, M, delta)
%
% [ind] = isinGT(gt, M, delta)
%
%  gt.......ground truth matches (mx2) [xl,xr]
%  M........matched points (nx2) [xl,xr]
%  delta...tolerance (default: 1 pixel)
%
%  ind.....index into G
%


if ~exist('delta','var'),
   delta = 1;
end

l = max([gt(:,1);M(:,1)]);
T1 = zeros(l,1);
T2 = zeros(l,1);

T1(gt(:,1))=gt(:,2);
T2(M(:,1))=M(:,2);
T2(T1==0)=0;

ind = find(abs(T1-T2)>delta);
return
% ----------------------------------------------
function [ind] = isinZone(p,zonetype,M)
%
% [ind] = isinZone(p,zonetype,M)
%
%  - p...one match 1x2 [xl,xr]
%  - zonetype...{'FX','X',delta} delta... interval (p-delta,p+delta)
%  - M..........matched points nx2 [xl,xr]
%
%  ->ind...indices into M of matches which
%          are in the forbidden zone of p
%

i = p(1);
j = p(2);

if strcmp(upper(zonetype),'X'),
   ind = find(M(:,1)==i | M(:,2)==j);  
elseif strcmp(upper(zonetype),'FX'),
   ind = find((M(:,1)>=i & M(:,2)<=j) | (M(:,1)<=i & M(:,2)>=j));
elseif ~ischar(zonetype) %delta   
   ind = find(M(:,1) == i & M(:,2)<=j+zonetype & M(:,2)>=j-zonetype);
else
   error('Invalid forbiden zone type.');
end

return
% ----------------------------------------------
function [ind] = FNf(P,zonetype,M);
%
% [ind] = FNf(P,zonetype,M); 
%
%  - P...ground truth matches (mx3) [xl,xr, ind_G]
%  - zonetype...{'FX','X'}
%  - M..........matched points (nx2) [xl,xr]
%
%  ->ind...indices into P of matches which
%          are considered as False negatives
%
%     [FNf: all p from P, Z(p) intersect M = 0 ] 
%

%in case the map is sub-pixel
P = round(P); %ground-truth 

ind=[];
for i = 1:size(P,1);
   if isempty(isinZone(P(i,1:2),zonetype,M)),
      ind(end+1)=P(i,3);
   end
end

return
% ----------------------------------------------
